// @flow

// TODO FEATURE support nested objects

'use strict';

// either true, or a string with the error
// use === true to test
Object.defineProperty(exports, '__esModule', {
  value: true
});
exports.create = create;
exports.createAll = createAll;
exports.validateExists = validateExists;
exports.validateTypeOf = validateTypeOf;
exports.validateInstanceOf = validateInstanceOf;
exports.validateRegex = validateRegex;

function _slicedToArray(arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i['return']) _i['return'](); } finally { if (_d) throw _e; } } return _arr; } else { throw new TypeError('Invalid attempt to destructure non-iterable instance'); } }

var _types = require('./types');

var _lodash = require('lodash');

// -------------------------------------------------------------

// create a single validator: a function to call with your object
// it will return an array of errors

function create(map, type) {
  var vs = typeValidators(map, type);
  return function (obj) {
    return validateAll(vs, obj);
  };
}

function createAll(map, types) {
  var vs = {};
  for (var name in types) {
    vs[name] = create(map, types[name]);
  }
  return vs;
}

// ---------------------------------------------------------------

var VALIDATORS_BY_TYPE = {
  'string': validateTypeOf('string'),
  'number': validateTypeOf('number'),
  'boolean': validateTypeOf('boolean'),
  'Date': validateInstanceOf(Date) };

function validateAll(vs, obj) {
  var maybeErrs = vs.map(function (kv) {
    return validate(kv, obj);
  });

  var errs = maybeErrs.filter(function (err) {
    return err !== undefined;
  });

  return errs;
}

function validate(_ref, obj) {
  var _ref2 = _slicedToArray(_ref, 2);

  var key = _ref2[0];
  var validator = _ref2[1];

  // this runs the validator
  var result = validator(obj[key]);
  if (!valid(result)) {
    return { key: key, value: obj[key], error: result };
  }
}

function valid(result) {
  return result === true;
}

//////////////////////////////////////////////////////////////////////////

// turns a property into a validator
// ignore optional, it doesn't work right
function propToValidator(map, prop) {
  return typeToValidator(map, prop.key, prop.type);
}

// just do required for now?
// you want to allow them to override the mapping
// especially for their custom types!
function typeToValidator(map, key, type) {

  // now run the type-based validator
  var validator = map[type.name];

  if (!validator) {
    throw new Error('Could not find validator for type: ' + type.name);
  }

  function isValid(value) {
    if (!exists(value)) {
      // if the property doesn't exist, and it's not a nullable property
      // otherwise just do the second one
      if (type.nullable) {
        return true;
      } else {
        return 'missing';
      }
    } else {
      return validator(value);
    }
  }

  return [key, isValid];
}

function typeValidators(map, type) {
  var fullMap = (0, _lodash.extend)(map, VALIDATORS_BY_TYPE);
  if (type.properties) {
    return objToValidators(fullMap, type.properties);
  } else {
    return [typeToValidator(map, '', type)];
  }
}

function objToValidators(map, props) {
  return props.map(function (prop) {
    return propToValidator(map, prop);
  });
}

//////////////////////////////////////////////////////////////
// Validators
//////////////////////////////////////////////////////////////

function validateExists() {
  return function (val) {
    return exists(val) || 'missing';
  };
}

function validateTypeOf(type) {
  return function (val) {
    return typeof val === type || 'expected typeof ' + type;
  };
}

function validateInstanceOf(type) {
  return function (val) {
    return val instanceof type || 'expected instance of ' + type;
  };
}

function validateRegex(regex) {
  return function (val) {
    return regex.test(val) || 'did not match ' + regex;
  };
}

function exists(value) {
  return !(value === undefined || value === null);
}